/**	
	class SetupEventsComponent : interface nsISetupEvent

	Description:	Act as a bridge between native XPCOM and 
					Javascript. This class help to achieve the 
					following: Javascript -> XPCOM(C++) -> Javascript
					
					Javascript -> XPCOM(C++) is easy to do, and widely documented
					
					XPCOM(C++) -> Javascript is more complex. It in theory requires
					to use JS_Call functions, but this is getting very tricky. Another
					option is to split this in 2 stages:
					
						XPCOM(C++) -> XPCOM(Javascript)
					then:
						XPCOM(Javascript) -> Javascript
					
					This is what this class is about. The extra benefit is that
					this class can be proxy'ed if we need to call Javascript
					from a working thread in the XPCOM component, thus leaving
					the UI Alive.

*/


// --- Class Identification ---
function SetupEventsComponent() {}


// -----------------------------------------------------------------------
// SetupEventsComponent Class itself...
// -----------------------------------------------------------------------
SetupEventsComponent.prototype = {

	mParent: null ,
	
	/** Need to store the window parent. This is called by the Javascript
		to store the window that will implement the callback.	

	*/
	SetParentDlg: function ( parent ) {		
		this.mParent = parent;
	} ,

	/** This method is called by the XPCOM(C++) when something 
		has to be reported to the UI */
	OnProgressText: function (aText) {
		if ( this.mParent == null ) {
			return;
		} else {
			this.mParent.OnProgressText( aText );
		}
	} ,
	
	/** This method is called by the XPCOM(C++) when something 
		has to be reported to the UI */
	OnProgressValue: function (val) {
		if ( this.mParent == null ) {
			return;
		} else {
			this.mParent.OnProgressValue( val );
		}
	} ,
	
	/** This method is called by the XPCOM(C++) when something 
		has to be reported to the UI */
	OnChildProgressText: function (aText) {
		if ( this.mParent == null ) {
			return;
		} else {
			this.mParent.OnChildProgressText( aText );
		}
	} ,
	
	/** This method is called by the XPCOM(C++) when something 
		has to be reported to the UI */
	OnChildProgressValue: function (val) {
		if ( this.mParent == null ) {
			return;
		} else {
			this.mParent.OnChildProgressValue( val );
		}
	} ,
	
	
	/** This method is called by the XPCOM(C++) when something 
		has to be reported to the UI */
	OnCompleteText: function (aValue) {
		if ( this.mParent == null ) {
			debug("Completed:" + aValue + "\n" );
		} else {
			this.mParent.OnCompleteText( aValue );
		}
	},
	
	/** This method is called by the XPCOM(C++) when something 
		has to be reported to the UI */
	OnCompleteCode: function (rv) {
		if ( this.mParent == null ) {
			debug("Completed:" + aValue + "\n" );
		} else {
			this.mParent.OnCompleteCode( rv );
		}
	},
	
	/** We need all the standard XPCOM boiler plate... in Javascript this time */
    QueryInterface: function (iid) {

        if (!iid.equals(Components.interfaces.nsISetupEvents) &&
            !iid.equals(Components.interfaces.nsISupports)) {
            debug(" in not an interface\n");
            throw Components.results.NS_ERROR_NO_INTERFACE;
        }
        debug(" is this\n");
        return this;
    } 	
}


// -----------------------------------------------------------------------
// Module and Factory Service...
// -----------------------------------------------------------------------
var module = {

    firstTime: true,

    /*
     * RegisterSelf is called at registration time (component installation
     * or the only-until-release startup autoregistration) and is responsible
     * for notifying the component manager of all components implemented in
     * this module.  The fileSpec, location and type parameters are mostly
     * opaque, and should be passed on to the registerComponent call
     * unmolested.
     */
    registerSelf: function (compMgr, fileSpec, location, type) {
        if (this.firstTime) {
            this.firstTime = false;
            throw Components.results.NS_ERROR_FACTORY_REGISTER_AGAIN;
        }
        
        compMgr = compMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);
        compMgr.registerFactoryLocation(this.myCID,
                                        "SetupEventsComponent JS Component",
                                        this.myProgID, 
                                        fileSpec,
                                        location , 
                                        type );
    },

    /*
     * The GetClassObject method is responsible for producing Factory and
     * SingletonFactory objects (the latter are specialized for services).
     */
    getClassObject: function (compMgr, cid, iid) {
    
        if (!cid.equals(this.myCID))
            throw Components.results.NS_ERROR_NO_INTERFACE;

        if (!iid.equals(Components.interfaces.nsIFactory))
            throw Components.results.NS_ERROR_NOT_IMPLEMENTED;

        return this.factory;
    },

    /* CID for this class */       
    myCID: Components.ID("{236d7f85-c302-468e-97db-79f77d75e8c2}"),

    /* ProgID for this class */
    myProgID: "@micromuse.com/NCISetupEvents;1",

    /* factory object */
    factory: {
        /*
         * Construct an instance of the interface specified by iid, possibly
         * aggregating it with the provided outer.  (If you don't know what
         * aggregation is all about, you don't need to.  It reduces even the
         * mightiest of XPCOM warriors to snivelling cowards.)
         */
        createInstance: function (outer, iid) {
            if (outer != null)
                throw Components.results.NS_ERROR_NO_AGGREGATION;

            return (new SetupEventsComponent()).QueryInterface(iid);
        }
    },

    /*
     * The canUnload method signals that the component is about to be unloaded.
     * C++ components can return false to indicate that they don't wish to be
     * unloaded, but the return value from JS components' canUnload is ignored:
     * mark-and-sweep will keep everything around until it's no longer in use,
     * making unconditional ``unload'' safe.
     *
     * You still need to provide a (likely useless) canUnload method, though:
     * it's part of the nsIModule interface contract, and the JS loader _will_
     * call it.
     */
    canUnload: function(compMgr) {
        return true;
    }
    
};

// --- module entry point ---
function NSGetModule(compMgr, fileSpec) {
    return module;
}

